@@ -26,6 +26,7 @@
 
 import datetime
 import sys
+import unittest
 
 import tests
 
@@ -82,6 +83,8 @@ FOREIGN KEY (id_t1) REFERENCES django_t1(id) ON DELETE CASCADE
 import django.db  # pylint: disable=W0611
 if tests.DJANGO_VERSION >= (1, 6):
     from django.db.backends import FieldInfo
+from django.db.backends.signals import connection_created
+from django.utils.safestring import SafeBytes, SafeText
 
 import mysql.connector
 from mysql.connector.django.base import (DatabaseWrapper, DatabaseOperations,
@@ -196,7 +199,7 @@ class DjangoDatabaseWrapper(tests.MySQLConnectorTests):
 
     def test__init__(self):
         exp = self.conn.get_server_version()
-        self.assertEqual(exp, self.cnx.server_version)
+        self.assertEqual(exp, self.cnx.mysql_version)
 
         value = datetime.time(2, 5, 7)
         exp = self.conn.converter._time_to_mysql(value)
@@ -207,6 +210,41 @@ class DjangoDatabaseWrapper(tests.MySQLConnectorTests):
         exp = self.conn.converter._time_to_mysql(value)
         self.assertEqual(exp, self.cnx.ops.value_to_db_time(value))
 
+    def test_signal(self):
+        from django.db import connection
+
+        def conn_setup(*args, **kwargs):
+            conn = kwargs['connection']
+            settings.DEBUG = True
+            cur = conn.cursor()
+            settings.DEBUG = False
+            cur.execute("SET @xyz=10")
+            cur.close()
+
+        connection_created.connect(conn_setup)
+        cursor = connection.cursor()
+        cursor.execute("SELECT @xyz")
+
+        self.assertEqual((10,), cursor.fetchone())
+        cursor.close()
+        self.cnx.close()
+
+    def count_conn(self, *args, **kwargs):
+        try:
+            self.connections += 1
+        except AttributeError:
+            self.connection = 1
+
+    def test_connections(self):
+        connection_created.connect(self.count_conn)
+        self.connections = 0
+
+        # Checking if DatabaseWrapper object creates a connection by default
+        conn = DatabaseWrapper(settings.DATABASES['default'])
+        dbo = DatabaseOperations(conn)
+        dbo.value_to_db_time(datetime.time(3, 3, 3))
+        self.assertEqual(self.connections, 0)
+
 
 class DjangoDatabaseOperations(tests.MySQLConnectorTests):
 
@@ -229,7 +267,7 @@ class DjangoDatabaseOperations(tests.MySQLConnectorTests):
         exp = self.conn.converter._time_to_mysql(value)
         self.assertEqual(exp, self.dbo.value_to_db_time(value))
 
-    def test_value_to_db_datetime(self, value):
+    def test_value_to_db_datetime(self):
         self.assertEqual(None, self.dbo.value_to_db_datetime(None))
 
         value = datetime.datetime(1, 1, 1)
@@ -249,4 +287,37 @@ class DjangoMySQLConverterTests(tests.MySQLConnectorTests):
         self.assertEqual(datetime.time(10, 11, 12),
                          django_converter._TIME_to_python(value, dsc=None))
 
+    def test__DATETIME_to_python(self):
+        value = b'1990-11-12 00:00:00'
+        django_converter = DjangoMySQLConverter()
+        self.assertEqual(datetime.datetime(1990, 11, 12, 0, 0, 0),
+                         django_converter._DATETIME_to_python(value, dsc=None))
 
+        settings.USE_TZ = True
+        value = b'0000-00-00 00:00:00'
+        django_converter = DjangoMySQLConverter()
+        self.assertEqual(None,
+                         django_converter._DATETIME_to_python(value, dsc=None))
+        settings.USE_TZ = False
+
+
+class BugOra20106629(tests.MySQLConnectorTests):
+    """CONNECTOR/PYTHON DJANGO BACKEND DOESN'T SUPPORT SAFETEXT"""
+    def setUp(self):
+        dbconfig = tests.get_mysql_config()
+        self.conn = mysql.connector.connect(**dbconfig)
+        self.cnx = DatabaseWrapper(settings.DATABASES['default'])
+        self.cur = self.cnx.cursor()
+        self.tbl = "BugOra20106629"
+        self.cur.execute("DROP TABLE IF EXISTS {0}".format(self.tbl), ())
+        self.cur.execute("CREATE TABLE {0}(col1 TEXT, col2 BLOB)".format(self.tbl), ())
+
+    def teardown(self):
+        self.cur.execute("DROP TABLE IF EXISTS {0}".format(self.tbl), ())
+
+    def test_safe_string(self):
+        safe_text = SafeText("dummy & safe data <html> ")
+        safe_bytes = SafeBytes(b"\x00\x00\x4c\x6e\x67\x39")
+        self.cur.execute("INSERT INTO {0} VALUES(%s, %s)".format(self.tbl), (safe_text, safe_bytes))
+        self.cur.execute("SELECT * FROM {0}".format(self.tbl), ())
+        self.assertEqual(self.cur.fetchall(), [(safe_text, safe_bytes)])
