@@ -30,8 +30,7 @@ options, simply do:
  shell> python unittests.py --help
 
 The unittest.py script will check for tests in Python source files prefixed
-with 'test_' in the folder tests/. Tests and other data specific to a
-Python major version are stored in tests/py2 and tests/py3.
+with 'test_' in the folder tests/.
 
 Examples:
  Running unit tests using MySQL installed under /opt
@@ -60,10 +59,16 @@ import sys
 import os
 import time
 import unittest
+try:
+    from urlparse import urlsplit
+except ImportError:
+    # Python 3
+    from urllib.parse import urlsplit
 import logging
+
 try:
     from argparse import ArgumentParser
-except:
+except ImportError:
     # Python v2.6
     from optparse import OptionParser
 
@@ -73,18 +78,24 @@ except ImportError:
     # Compatibility with Python v2.6
     from unittest import _TextTestResult as TextTestResult
 
-if ((sys.version_info >= (2, 6) and sys.version_info < (3, 0))
-        or sys.version_info >= (3, 1)):
-    sys.path.insert(0, 'python{0}/'.format(sys.version_info[0]))
-else:
-    raise RuntimeError("Python v%d.%d is not supported" %
-                       sys.version_info[0:2])
-
-import mysql.connector
 import tests
 from tests import mysqld
 
+_TOPDIR = os.path.dirname(os.path.realpath(__file__))
 LOGGER = logging.getLogger(tests.LOGGER_NAME)
+tests.setup_logger(LOGGER)
+
+# Only run for supported Python Versions
+if not (((2, 6) <= sys.version_info < (3, 0)) or sys.version_info >= (3, 3)):
+    LOGGER.error("Python v%d.%d is not supported",
+                 sys.version_info[0], sys.version_info[1])
+    sys.exit(1)
+else:
+    sys.path.insert(0, os.path.join(_TOPDIR, 'lib'))
+    sys.path.insert(0, os.path.join(_TOPDIR))
+
+import mysql.connector
+
 
 # MySQL option file template. Platform specifics dynamically added later.
 MY_CNF = """
@@ -94,18 +105,22 @@ innodb_compression_level = 0
 innodb_compression_failure_threshold_pct = 0
 lc_messages_dir = {lc_messages_dir}
 lc_messages = en_US
+general_log = ON
 
 [mysqld-5.5]
 lc_messages_dir = {lc_messages_dir}
 lc_messages = en_US
+general_log = ON
 
 [mysqld-5.1]
-language = {lc_messages_dir}
+language = {lc_messages_dir}/english
+general_log = ON
 
 [mysqld-5.0]
-language = {lc_messages_dir}
+language = {lc_messages_dir}/english
 
 [mysqld]
+max_allowed_packet=26777216
 basedir = {basedir}
 datadir = {datadir}
 tmpdir = {tmpdir}
@@ -119,9 +134,10 @@ sql_mode = ""
 default_time_zone = +00:00
 log-error = mysqld_{name}.err
 log-bin = mysqld_{name}_bin
-general_log = ON
 local_infile = 1
 innodb_flush_log_at_trx_commit = 2
+innodb_log_file_size = 1Gb
+general_log_file = general_{name}.log
 ssl
 """
 
@@ -143,7 +159,7 @@ else:
     ))
     MYSQL_DEFAULT_BASE = os.path.join('/', 'usr', 'local', 'mysql')
 
-MYSQL_DEFAULT_TOPDIR = os.path.dirname(os.path.realpath(__file__))
+MYSQL_DEFAULT_TOPDIR = _TOPDIR
 
 _UNITTESTS_CMD_ARGS = {
     ('-T', '--one-test'): {
@@ -236,6 +252,13 @@ _UNITTESTS_CMD_ARGS = {
             "(default {default})").format(default=MYSQL_DEFAULT_BASE)
     },
 
+    ('', '--with-mysql-share'): {
+        'dest': 'mysql_sharedir', 'metavar': 'NAME',
+        'default': None,
+        'help': (
+            "share folder of the MySQL server (default <basedir>/share)")
+    },
+
     ('', '--mysql-topdir'): {
         'dest': 'mysql_topdir', 'metavar': 'NAME',
         'default': MYSQL_DEFAULT_TOPDIR,
@@ -271,23 +294,36 @@ _UNITTESTS_CMD_ARGS = {
             'Use IPv6 to run tests. This sets --bind-address=:: --host=::1.'
         ),
     },
+
+    ('', '--with-django'): {
+        'dest': 'django_path', 'metavar': 'NAME',
+        'default': None,
+        'help': ("Location of Django (none installed source)")
+    },
+
+    ('', '--with-fabric'): {
+        'dest': 'fabric_config', 'metavar': 'NAME',
+        'default': None,
+        'help': ("Fabric configuration as URI fabric://user:pass@server:port")
+    },
 }
 
 
 def _get_arg_parser():
-    """Parse comand line ArgumentParser
+    """Parse command line ArgumentParser
 
     This function parses the command line arguments and returns the parser.
 
     It works with both optparse and argparse where available.
     """
+
     def _clean_optparse(adict):
         """Remove items from dictionary ending with _optparse"""
-        new = {}
+        new_dict = {}
         for key in adict.keys():
             if not key.endswith('_optparse'):
-                new[key] = adict[key]
-        return new
+                new_dict[key] = adict[key]
+        return new_dict
 
     new = True
     try:
@@ -337,7 +373,6 @@ def get_stats_field(pyver=None, myver=None):
 
 
 class StatsTestResult(TextTestResult):
-
     """Store test results in a database"""
     separator1 = '=' * 78
     separator2 = '-' * 78
@@ -354,7 +389,10 @@ class StatsTestResult(TextTestResult):
         self._dbcnx = dbcnx
         self._name = None
 
-    def get_description(self, test):  # pylint: disable=R0201
+    @staticmethod
+    def get_description(test):  # pylint: disable=R0201
+        """Return test description, if needed truncated to 60 characters
+        """
         return "{0:.<60s} ".format(str(test)[0:58])
 
     def startTest(self, test):
@@ -377,8 +415,8 @@ class StatsTestResult(TextTestResult):
             ).format(table=get_stats_tablename(),
                      field=get_stats_field())
             cur.execute(stmt,
-                       (str(test), self.elapsed_time, self.elapsed_time)
-                        )
+                        (str(test), self.elapsed_time, self.elapsed_time)
+            )
             cur.close()
 
     def _save_not_ok(self, test):
@@ -427,7 +465,6 @@ class StatsTestResult(TextTestResult):
 
 
 class StatsTestRunner(unittest.TextTestRunner):
-
     """Committing results test results"""
     resultclass = StatsTestResult
 
@@ -455,7 +492,6 @@ class StatsTestRunner(unittest.TextTestRunner):
 
 
 class BasicTestResult(TextTestResult):
-
     """Basic test result"""
 
     def addSkip(self, test, reason):
@@ -464,16 +500,16 @@ class BasicTestResult(TextTestResult):
 
 
 class BasicTestRunner(unittest.TextTestRunner):
-
     """Basic test runner"""
     resultclass = BasicTestResult
 
     def __init__(self, stream=sys.stderr, descriptions=True, verbosity=1,
-                 failfast=False, buffer=False):
+                 failfast=False, buffer=False, warnings='ignore'):
         try:
             super(BasicTestRunner, self).__init__(
                 stream=stream, descriptions=descriptions,
-                verbosity=verbosity, failfast=failfast, buffer=buffer)
+                verbosity=verbosity, failfast=failfast, buffer=buffer,
+                warnings=warnings)
         except TypeError:
             # Python v3.1
             super(BasicTestRunner, self).__init__(
@@ -481,7 +517,6 @@ class BasicTestRunner(unittest.TextTestRunner):
 
 
 class Python26TestRunner(unittest.TextTestRunner):
-
     """Python v2.6/3.1 Test Runner backporting needed functionality"""
 
     def __init__(self, stream=sys.stderr, descriptions=True, verbosity=1,
@@ -544,11 +579,17 @@ def init_mysql_server(port, options):
             port=port,
             unix_socket_folder=options.unix_socket_folder,
             ssl_folder=os.path.abspath(tests.SSL_DIR),
-            name=name)
+            name=name,
+            sharedir=options.mysql_sharedir)
     except tests.mysqld.MySQLBootstrapError as err:
         LOGGER.error("Failed initializing MySQL server "
-                     "'{name}': {error} (use --with-mysql?)".format(
-                         name=name, error=str(err)))
+                     "'{name}': {error}".format(
+            name=name, error=str(err)))
+        sys.exit(1)
+
+    if len(mysql_server.unix_socket) > 103:
+        LOGGER.error("Unix socket file is to long for mysqld (>103). "
+                     "Consider using --unix-socket")
         sys.exit(1)
 
     mysql_server._debug = options.debug
@@ -578,16 +619,17 @@ def init_mysql_server(port, options):
             mysql_server.remove()
 
     tests.MYSQL_VERSION = mysql_server.version
+    tests.MYSQL_VERSION_TXT = '.'.join([str(i) for i in mysql_server.version])
     tests.MYSQL_SERVERS.append(mysql_server)
 
     mysql_server.client_config = {
         'host': options.host,
         'port': port,
-        'unix_socket': mysql_server._unix_socket,
+        'unix_socket': mysql_server.unix_socket,
         'user': 'root',
         'password': '',
         'database': 'myconnpy',
-        'connection_timeout': 5,
+        'connection_timeout': 10,
     }
 
     # Bootstrap and start a MySQL server
@@ -595,9 +637,9 @@ def init_mysql_server(port, options):
         LOGGER.info("Bootstrapping MySQL server '{name}'".format(name=name))
         try:
             mysql_server.bootstrap()
-        except tests.mysqld.MySQLBootstrapError:
-            LOGGER.error("Failed bootstrapping MySQL server '{name}'".format(
-                name=name))
+        except tests.mysqld.MySQLBootstrapError as exc:
+            LOGGER.error("Failed bootstrapping MySQL server '{name}': "
+                         "{error}".format(name=name, error=str(exc)))
             sys.exit(1)
         mysql_server.start()
         if not mysql_server.wait_up():
@@ -606,35 +648,18 @@ def init_mysql_server(port, options):
             sys.exit(1)
 
 
-def setup_logger(logger, debug=False, logfile=None):
-    """Setting up the logger"""
-    formatter = logging.Formatter(
-        "%(asctime)s [%(name)s:%(levelname)s] %(message)s")
-    handler = None
-    if logfile:
-        handler = logging.FileHandler(logfile)
-    else:
-        handler = logging.StreamHandler()
-    handler.setFormatter(formatter)
-    if debug:
-        logger.setLevel(logging.DEBUG)
-    else:
-        logger.setLevel(logging.INFO)
-    LOGGER.addHandler(handler)
-
-
 def main():
     parser = _get_arg_parser()
     options = parser.parse_args()
+    tests.OPTIONS_INIT = True
 
     if isinstance(options, tuple):
         # Fallback to old optparse
         options = options[0]
 
-    setup_logger(LOGGER, debug=options.debug, logfile=options.logfile)
+    tests.setup_logger(LOGGER, debug=options.debug, logfile=options.logfile)
     LOGGER.info(
-        "MySQL Connector/Python unittest "
-        "started using Python v{0}".format(
+        "MySQL Connector/Python unittest using Python v{0}".format(
             '.'.join([str(v) for v in sys.version_info[0:3]])))
 
     # Check if we can test IPv6
@@ -648,7 +673,58 @@ def main():
     else:
         tests.IPV6_AVAILABLE = False
 
+    if not options.mysql_sharedir:
+        options.mysql_sharedir = os.path.join(options.mysql_basedir, 'share')
+        LOGGER.debug("Setting default sharedir: %s", options.mysql_sharedir)
+    if options.mysql_topdir != MYSQL_DEFAULT_TOPDIR:
+        # Make sure the topdir is absolute
+        if not os.path.isabs(options.mysql_topdir):
+            options.mysql_topdir = os.path.join(
+                os.path.dirname(os.path.realpath(__file__)),
+                options.mysql_topdir
+            )
+
+    # If Django was supplied, add Django to PYTHONPATH
+    if options.django_path:
+        sys.path.insert(0, options.django_path)
+        try:
+            import django
+
+            tests.DJANGO_VERSION = django.VERSION[0:3]
+        except ImportError:
+            msg = "Could not find django package at {0}".format(
+                options.django_path)
+            LOGGER.error(msg)
+            sys.exit(1)
+
+        if sys.version_info[0] == 3 and tests.DJANGO_VERSION < (1, 5):
+            LOGGER.error("Django older than v1.5 will not work with Python 3")
+            sys.exit(1)
+
+    if options.fabric_config:
+        # URL example: fabric://user:pass@mysqlfabric.example.com:32274
+        fab = urlsplit(options.fabric_config)
+        tests.FABRIC_CONFIG = {
+            'host': fab.hostname,
+            'port': fab.port or 32274,
+            'user': fab.username,
+            'password': fab.password,
+        }
+
+    # Start Dummy MySQL Server
+    #tests.MYSQL_DUMMY = mysqld.DummyMySQLServer(('127.0.0.1', options.port - 1),
+    #                                            mysqld.DummyMySQLRequestHandler)
+    #tests.MYSQL_DUMMY_THREAD = threading.Thread(
+    #    target=tests.MYSQL_DUMMY.serve_forever)
+    #tests.MYSQL_DUMMY_THREAD.setDaemon(True)
+
+    # We have to at least run 1 MySQL server
+    init_mysql_server(port=(options.port), options=options)
+
     # Which tests cases to run
+    testcases = []
+    testsuite = None
+
     if options.testcase:
         if options.testcase in tests.get_test_names():
             for module in tests.get_test_modules():
@@ -666,13 +742,12 @@ def main():
         testcases = tests.get_test_modules()
         testsuite = unittest.TestLoader().loadTestsFromNames(testcases)
 
-    # Initialize the MySQL Servers
-    for i in range(0, tests.MYSQL_SERVERS_NEEDED):
+    # Initialize the other MySQL Servers
+    for i in range(1, tests.MYSQL_SERVERS_NEEDED):
         init_mysql_server(port=(options.port + i), options=options)
 
-    LOGGER.info(
-        "Using MySQL server version {0}".format(
-            '.'.join([str(v) for v in tests.MYSQL_VERSION[0:3]])))
+    LOGGER.info("Using MySQL server version %s",
+                '.'.join([str(v) for v in tests.MYSQL_VERSION[0:3]]))
 
     LOGGER.info("Starting unit tests")
     was_successful = False
@@ -692,14 +767,12 @@ def main():
             else:
                 cnxstats = None
             result = StatsTestRunner(
-                verbosity=options.verbosity, dbcnx=cnxstats).run(
-                testsuite)
+                verbosity=options.verbosity, dbcnx=cnxstats).run(testsuite)
         elif sys.version_info[0:2] == (2, 6):
             result = Python26TestRunner(verbosity=options.verbosity).run(
                 testsuite)
         else:
-            result = BasicTestRunner(verbosity=options.verbosity).run(
-                testsuite)
+            result = BasicTestRunner(verbosity=options.verbosity).run(testsuite)
         was_successful = result.wasSuccessful()
     except KeyboardInterrupt:
         LOGGER.info("Unittesting was interrupted")
@@ -713,46 +786,52 @@ def main():
 
     # Show skipped tests
     if len(tests.MESSAGES['SKIPPED']):
-        LOGGER.info(
-            "Skipped tests: {0}".format(
-                len(tests.MESSAGES['SKIPPED'])))
-        if options.verbosity >= 1 or options.debug:
-            for msg in tests.MESSAGES['SKIPPED']:
-                LOGGER.info(msg)
+        LOGGER.info("Skipped tests: %d", len(tests.MESSAGES['SKIPPED']))
+        for msg in tests.MESSAGES['SKIPPED']:
+            LOGGER.info("Skipped: " + msg)
 
     # Clean up
+    try:
+        tests.MYSQL_DUMMY_THREAD.join()
+        tests.MYSQL_DUMMY.shutdown()
+        tests.MYSQL_DUMMY.server_close()
+    except:
+        # Is OK when failed
+        pass
     for mysql_server in tests.MYSQL_SERVERS:
         name = mysql_server.name
         if not options.keep:
             mysql_server.stop()
             if not mysql_server.wait_down():
-                LOGGER.error("Failed stopping MySQL server '{name}'".format(
-                    name=name))
+                LOGGER.error("Failed stopping MySQL server '%s'", name)
             else:
                 mysql_server.remove()
-                LOGGER.info(
-                    "MySQL server '{name}' stopped and cleaned up".format(
-                        name=name))
+                LOGGER.info("MySQL server '%s' stopped and cleaned up", name)
         elif not mysql_server.check_running():
             mysql_server.start()
             if not mysql_server.wait_up():
                 LOGGER.error("MySQL could not be kept running; "
                              "failed to restart")
         else:
-            LOGGER.info("MySQL server kept running on {addr}:{port}".format(
-                addr=mysql_server.bind_address,
-                port=mysql_server.port)
+            LOGGER.info("MySQL server kept running on %s:%d",
+                        mysql_server.bind_address,
+                        mysql_server.port
             )
 
+    # Make sure the DEVNULL file is closed
+    try:
+        mysqld.DEVNULL.close()
+    except:
+        pass
+
     txt = ""
     if not was_successful:
         txt = "not "
-    LOGGER.info(
-        "MySQL Connector/Python unittests were {result}successful".format(
-            result=txt))
+    LOGGER.info("MySQL Connector/Python unittests were %ssuccessful", txt)
 
     # Return result of tests as exit code
     sys.exit(not was_successful)
 
+
 if __name__ == '__main__':
     main()
